@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2015 DNAnexus, Inc.
+# Copyright (C) 2013-2016 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -38,9 +38,14 @@ from __future__ import print_function, unicode_literals, division, absolute_impo
 
 import os, sys, json, subprocess, tempfile, multiprocessing
 import datetime
+import gzip
 import hashlib
+import io
+import tarfile
+import stat
 
 import dxpy
+import dxpy.executable_builder
 from . import logger
 from .utils import merge
 from .utils.printing import fill
@@ -49,8 +54,9 @@ from .cli import INTERACTIVE_CLI
 
 NUM_CORES = multiprocessing.cpu_count()
 
-DX_TOOLKIT_PKGS = ['dx-toolkit', 'dx-toolkit-beta', 'dx-toolkit-unstable']
-DX_TOOLKIT_GIT_URLS = ["git@github.com:dnanexus/dx-toolkit.git"]
+DX_TOOLKIT_PKGS = ('dx-toolkit',)
+DX_TOOLKIT_GIT_URLS = ("git@github.com:dnanexus/dx-toolkit.git",)
+
 
 class AppBuilderException(Exception):
     """
@@ -131,58 +137,62 @@ def get_destination_project(src_dir, project=None):
         return project
     return _get_applet_spec(src_dir)['project']
 
-
-def _directory_checksum(resources_dir):
-    """Returns a hash of the contents of resources_dir.
-
-    The hash is based purely on the names, mtimes, and modes of the
-    files, and not on the contents or other metadata.
-
+def is_link_local(link_target):
     """
+    :param link_target: The target of a symbolic link, as given by os.readlink()
+    :type link_target: string
+    :returns: A boolean indicating the link is local to the current directory.
+              This is defined to mean that os.path.isabs(link_target) == False
+              and the link NEVER references the parent directory, so
+              "./foo/../../curdir/foo" would return False.
+    :rtype: boolean
+    """
+    is_local=(not os.path.isabs(link_target))
 
-    # The input to the SHA1 contains entries of the form (whitespace
-    # only included here for readability):
-    #
-    # / \0 MODE \0 MTIME \0
-    # /foo \0 MODE \0 MTIME \0
-    # ...
-    #
-    # where there is one entry for each directory or file (order is
-    # specified below), followed by a numeric representation of the
-    # mode, and the mtime in milliseconds since the epoch.
+    if is_local:
+        # make sure that the path NEVER extends outside the resources directory!
+        d,l = os.path.split(link_target)
+        link_parts = []
+        while l:
+            link_parts.append(l)
+            d,l = os.path.split(d)
+        curr_path = os.sep
 
-    output_sha1 = hashlib.sha1()
+        for p in reversed(link_parts):
+            is_local = (is_local and not (curr_path == os.sep and p == os.pardir) )
+            curr_path = os.path.abspath(os.path.join(curr_path, p))
 
-    for dirname, subdirs, files in os.walk(resources_dir):
-        if not dirname.startswith(resources_dir):
-            raise AssertionError('Expected %r to start with root directory %r' % (dirname, resources_dir))
+    return is_local
 
-        # Add an entry for the directory itself
-        relative_dirname = dirname[len(resources_dir):]
-        dir_stat = os.lstat(dirname)
-        if not relative_dirname.startswith('/'):
-            relative_dirname = '/' + relative_dirname
-        fields = [relative_dirname, str(dir_stat.st_mode), str(int(dir_stat.st_mtime * 1000))]
-        output_sha1.update(b''.join(s.encode('utf-8') + b'\0' for s in fields))
+def _fix_perms(perm_obj):
+    """
+    :param perm_obj: A permissions object, as given by os.stat()
+    :type perm_obj: integer
+    :returns: A permissions object that is the result of "chmod a+rX" on the
+              given permission object.  This is defined to be the permission object
+              bitwise or-ed with all stat.S_IR*, and if the stat.S_IXUSR bit is
+              set, then the permission object should also be returned bitwise or-ed
+              with stat.S_IX* (stat.S_IXUSR not included because it would be redundant).
+    :rtype: integer
+    """
+    ret_perm = perm_obj | stat.S_IROTH | stat.S_IRGRP | stat.S_IRUSR
+    if ret_perm & stat.S_IXUSR:
+        ret_perm = ret_perm | stat.S_IXGRP | stat.S_IXOTH
 
-        # Canonicalize the order of files so that we compute the
-        # checksum in a consistent order
-        for filename in sorted(files):
-            relative_filename = os.path.join(relative_dirname, filename)
-            true_filename = os.path.join(dirname, filename)
+    return ret_perm
 
-            file_stat = os.lstat(true_filename)
-            fields = [relative_filename, str(file_stat.st_mode), str(int(file_stat.st_mtime * 1000))]
-            output_sha1.update(b''.join(s.encode('utf-8') + b'\0' for s in fields))
-
-        # Canonicalize the order of subdirectories; this is the order in
-        # which they will be visited by os.walk
-        subdirs.sort()
-
-    return output_sha1.hexdigest()
+def _fix_perm_filter(tar_obj):
+    """
+    :param tar_obj: A TarInfo object to be added to a tar file
+    :tpye tar_obj: tarfile.TarInfo
+    :returns: A TarInfo object with permissions changed (a+rX)
+    :rtype: tarfile.TarInfo
+    """
+    tar_obj.mode = _fix_perms(tar_obj.mode)
+    return tar_obj
 
 
-def upload_resources(src_dir, project=None, folder='/', ensure_upload=False):
+def upload_resources(src_dir, project=None, folder='/', ensure_upload=False, force_symlinks=False):
     """
     :param ensure_upload: If True, will bypass checksum of resources directory
                           and upload resources bundle unconditionally;
@@ -191,6 +201,12 @@ def upload_resources(src_dir, project=None, folder='/', ensure_upload=False):
                           if checksum is different from a previously uploaded
                           bundle's checksum.
     :type ensure_upload: boolean
+    :param force_symlinks: If true, will bypass the attempt to dereference any
+                           non-local symlinks and will unconditionally include
+                           the link as-is.  Note that this will almost certainly
+                           result in a broken link within the resource directory
+                           unless you really know what you're doing.
+    :type force_symlinks: boolean
     :returns: A list (possibly empty) of references to the generated archive(s)
     :rtype: list
 
@@ -212,50 +228,182 @@ def upload_resources(src_dir, project=None, folder='/', ensure_upload=False):
     if os.path.exists(resources_dir) and len(os.listdir(resources_dir)) > 0:
         target_folder = applet_spec['folder'] if 'folder' in applet_spec else folder
 
-        # Optimistically look for a resource bundle with the same
-        # contents, and reuse it if possible. The resource bundle
-        # carries a property 'resource_bundle_checksum' that indicates
-        # the checksum; the way in which the checksum is computed is
-        # given in the documentation of _directory_checksum.
+        # While creating the resource bundle, optimistically look for a
+        # resource bundle with the same contents, and reuse it if possible.
+        # The resource bundle carries a property 'resource_bundle_checksum'
+        # that indicates the checksum; the way in which the checksum is
+        # computed is given below.   If the checksum matches  (and
+        # ensure_upload is False), then we will use the existing file,
+        # otherwise, we will compress and upload the tarball.
 
-        if ensure_upload:
-            properties_dict = {}
-            existing_resources = False
-        else:
-            directory_checksum = _directory_checksum(resources_dir)
-            properties_dict = dict(resource_bundle_checksum=directory_checksum)
-            existing_resources = dxpy.find_one_data_object(
-                project=dest_project,
-                folder=target_folder,
-                properties=dict(resource_bundle_checksum=directory_checksum),
-                visibility='either',
-                zero_ok=True,
-                state='closed',
-                return_handler=True
-            )
 
-        if existing_resources:
-            logger.info("Found existing resource bundle that matches local resources directory: " +
-                        existing_resources.get_id())
+        # The input to the SHA1 contains entries of the form (whitespace
+        # only included here for readability):
+        #
+        # / \0 MODE \0 MTIME \0
+        # /foo \0 MODE \0 MTIME \0
+        # ...
+        #
+        # where there is one entry for each directory or file (order is
+        # specified below), followed by a numeric representation of the
+        # mode, and the mtime in milliseconds since the epoch.
+        #
+        # Note when looking at a link, if the link is to be dereferenced,
+        # the mtime and mode used are that of the target (using os.stat())
+        # If the link is to be kept as a link, the mtime and mode are those
+        # of the link itself (using os.lstat())
 
-            dx_resource_archive = existing_resources
-        else:
-            logger.debug("Uploading in " + src_dir)
+        with tempfile.NamedTemporaryFile(suffix=".tar") as tar_tmp_fh:
+
+            output_sha1 = hashlib.sha1()
+            tar_fh = tarfile.open(fileobj=tar_tmp_fh, mode='w')
+
+            for dirname, subdirs, files in os.walk(resources_dir):
+                if not dirname.startswith(resources_dir):
+                    raise AssertionError('Expected %r to start with root directory %r' % (dirname, resources_dir))
+
+                # Add an entry for the directory itself
+                relative_dirname = dirname[len(resources_dir):]
+                dir_stat = os.lstat(dirname)
+                if not relative_dirname.startswith('/'):
+                    relative_dirname = '/' + relative_dirname
+
+                fields = [relative_dirname, str(_fix_perms(dir_stat.st_mode)), str(int(dir_stat.st_mtime * 1000))]
+                output_sha1.update(b''.join(s.encode('utf-8') + b'\0' for s in fields))
+
+                # add an entry in the tar file for the current directory, but
+                # do not recurse!
+                tar_fh.add(dirname, arcname='.' + relative_dirname, recursive=False, filter=_fix_perm_filter)
+
+                # Canonicalize the order of subdirectories; this is the order in
+                # which they will be visited by os.walk
+                subdirs.sort()
+
+                # check the subdirectories for symlinks.  We should throw an error
+                # if there are any links that point outside of the directory (unless
+                # --force-symlinks is given).  If a link is pointing internal to
+                # the directory (or --force-symlinks is given), we should add it
+                # as a file.
+                for subdir_name in subdirs:
+                    dir_path = os.path.join(dirname, subdir_name)
+
+                    # If we do have a symlink,
+                    if os.path.islink(dir_path):
+                        # Let's get the pointed-to path to ensure that it is
+                        # still in the directory
+                        link_target = os.readlink(dir_path)
+
+                        # If this is a local link, add it to the list of files (case 1)
+                        # else raise an error
+                        if force_symlinks or is_link_local(link_target):
+                            files.append(subdir_name)
+                        else:
+                            raise AppBuilderException("Cannot include symlinks to directories outside of the resource directory.  '%s' points to directory '%s'" % (dir_path, os.path.realpath(dir_path)))
+
+
+                # Canonicalize the order of files so that we compute the
+                # checksum in a consistent order
+                for filename in sorted(files):
+                    deref_link = False
+
+                    relative_filename = os.path.join(relative_dirname, filename)
+                    true_filename = os.path.join(dirname, filename)
+
+                    file_stat = os.lstat(true_filename)
+                    # check for a link here, please!
+                    if os.path.islink(true_filename):
+
+                        # Get the pointed-to path
+                        link_target = os.readlink(true_filename)
+
+                        if not (force_symlinks or is_link_local(link_target)):
+                            # if we are pointing outside of the directory, then:
+                            # try to get the true stat of the file and make sure
+                            # to dereference the link!
+                            try:
+                                file_stat = os.stat(os.path.join(dirname, link_target))
+                                deref_link = True
+                            except OSError:
+                                # uh-oh! looks like we have a broken link!
+                                # since this is guaranteed to cause problems (and
+                                # we know we're not forcing symlinks here), we
+                                # should throw an error
+                                raise AppBuilderException("Broken symlink: Link '%s' points to '%s', which does not exist" % (true_filename, os.path.realpath(true_filename)) )
+
+
+                    fields = [relative_filename, str(_fix_perms(file_stat.st_mode)), str(int(file_stat.st_mtime * 1000))]
+                    output_sha1.update(b''.join(s.encode('utf-8') + b'\0' for s in fields))
+
+                    # If we are to dereference, use the target fn
+                    if deref_link:
+                        true_filename = os.path.realpath(true_filename)
+
+                    tar_fh.add(true_filename, arcname='.' + relative_filename, filter=_fix_perm_filter)
+
+                # end for filename in sorted(files)
+
+            # end for dirname, subdirs, files in os.walk(resources_dir):
+
+            # at this point, the tar is complete, so close the tar_fh
+            tar_fh.close()
+
+            # Optimistically look for a resource bundle with the same
+            # contents, and reuse it if possible. The resource bundle
+            # carries a property 'resource_bundle_checksum' that indicates
+            # the checksum; the way in which the checksum is computed is
+            # given in the documentation of _directory_checksum.
+
+            if ensure_upload:
+                properties_dict = {}
+                existing_resources = False
+            else:
+                directory_checksum = output_sha1.hexdigest()
+                properties_dict = dict(resource_bundle_checksum=directory_checksum)
+                existing_resources = dxpy.find_one_data_object(
+                    project=dest_project,
+                    folder=target_folder,
+                    properties=dict(resource_bundle_checksum=directory_checksum),
+                    visibility='either',
+                    zero_ok=True,
+                    state='closed',
+                    return_handler=True
+                )
+
+            if existing_resources:
+                logger.info("Found existing resource bundle that matches local resources directory: " +
+                            existing_resources.get_id())
+
+                dx_resource_archive = existing_resources
+            else:
+
+                logger.debug("Uploading in " + src_dir)
+                # We need to compress the tar that we've created
+
+
+                targz_fh = tempfile.NamedTemporaryFile(suffix=".tar.gz", delete=False)
+
+                # compress the file by reading the tar file and passing
+                # it though a GzipFile object, writing the given
+                # block size (by default 8192 bytes) at a time
+                targz_gzf = gzip.GzipFile(fileobj=targz_fh)
+                tar_tmp_fh.seek(0)
+                dat = tar_tmp_fh.read(io.DEFAULT_BUFFER_SIZE)
+                while dat:
+                    targz_gzf.write(dat)
+                    dat = tar_tmp_fh.read(io.DEFAULT_BUFFER_SIZE)
+
+                targz_gzf.flush()
+                targz_gzf.close()
+                targz_fh.close()
 
-            with tempfile.NamedTemporaryFile(suffix=".tar.gz") as tar_fh:
-                # The directory contents may have changed since the
-                # first time we checksummed the directory. Ideally we
-                # would extract the tar file to determine the checksum
-                # of the actually archived files, but maybe this is a
-                # little too paranoid.
-                subprocess.check_call(['tar', '-C', resources_dir, '-czf', tar_fh.name, '.'])
                 if 'folder' in applet_spec:
                     try:
                         dxpy.get_handler(dest_project).new_folder(applet_spec['folder'], parents=True)
                     except dxpy.exceptions.DXAPIError:
                         pass # TODO: make this better
+
                 dx_resource_archive = dxpy.upload_local_file(
-                    tar_fh.name,
+                    targz_fh.name,
                     wait_on_close=True,
                     project=dest_project,
                     folder=target_folder,
@@ -263,35 +411,18 @@ def upload_resources(src_dir, project=None, folder='/', ensure_upload=False):
                     properties=properties_dict
                 )
 
-        archive_link = dxpy.dxlink(dx_resource_archive.get_id())
+                os.unlink(targz_fh.name)
+
+                # end compressed file creation and upload
+
+            archive_link = dxpy.dxlink(dx_resource_archive.get_id())
+
+        # end tempfile.NamedTemporaryFile(suffix=".tar") as tar_fh
+
         return [{'name': 'resources.tar.gz', 'id': archive_link}]
     else:
         return []
 
-def _inline_documentation_files(app_spec, src_dir):
-    """
-    Modifies the provided app_spec dict (which may be an app or applet
-    spec, actually) to inline the contents of the readme file into
-    "description" and the developer readme into "developerNotes".
-    """
-    # Inline description from a readme file
-    if 'description' not in app_spec:
-        readme_filename = None
-        for filename in 'README.md', 'Readme.md', 'readme.md':
-            if os.path.exists(os.path.join(src_dir, filename)):
-                readme_filename = filename
-                break
-        if readme_filename is not None:
-            with open(os.path.join(src_dir, readme_filename)) as fh:
-                app_spec['description'] = fh.read()
-
-    # Inline developerNotes from Readme.developer.md
-    if 'developerNotes' not in app_spec:
-        for filename in 'README.developer.md', 'Readme.developer.md', 'readme.developer.md':
-            if os.path.exists(os.path.join(src_dir, filename)):
-                with open(os.path.join(src_dir, filename)) as fh:
-                    app_spec['developerNotes'] = fh.read()
-                break
 
 def upload_applet(src_dir, uploaded_resources, check_name_collisions=True, overwrite=False, archive=False, project=None, override_folder=None, override_name=None, dx_toolkit_autodep="stable", dry_run=False, **kwargs):
     """
@@ -303,8 +434,12 @@ def upload_applet(src_dir, uploaded_resources, check_name_collisions=True, overw
     :type override_folder: str
     :param override_name: name for the resulting applet which, if specified, overrides that given in dxapp.json
     :type override_name: str
-    :param dx_toolkit_autodep: What type of dx-toolkit dependency to inject if none is present. "stable" for the APT package; "git" for HEAD of dx-toolkit master branch; or False for no dependency.
+    :param dx_toolkit_autodep: What type of dx-toolkit dependency to
+        inject if none is present. "stable" for the APT package; "git"
+        for HEAD of dx-toolkit master branch; or False for no
+        dependency.
     :type dx_toolkit_autodep: boolean or string
+
     """
     applet_spec = _get_applet_spec(src_dir)
 
@@ -331,7 +466,7 @@ def upload_applet(src_dir, uploaded_resources, check_name_collisions=True, overw
     if 'dxapi' not in applet_spec:
         applet_spec['dxapi'] = dxpy.API_VERSION
 
-    applet_to_overwrite = None
+    applets_to_overwrite = []
     archived_applet = None
     if check_name_collisions and not dry_run:
         destination_path = applet_spec['folder'] + ('/' if not applet_spec['folder'].endswith('/') else '') + applet_spec['name']
@@ -343,7 +478,7 @@ def upload_applet(src_dir, uploaded_resources, check_name_collisions=True, overw
                 # we remove the old applet, but that causes garbage
                 # collection of the bundled resources that will be
                 # shared with the new applet
-                applet_to_overwrite = result['id']
+                applets_to_overwrite.append(result['id'])
             elif archive:
                 logger.debug("Archiving applet %s" % (result['id']))
                 proj = dxpy.DXProject(dest_project)
@@ -365,25 +500,112 @@ def upload_applet(src_dir, uploaded_resources, check_name_collisions=True, overw
     # -----
     # Override various fields from the pristine dxapp.json
 
+    # Carry region-specific values from regionalOptions into the main
+    # runSpec
+    applet_spec["runSpec"].setdefault("bundledDepends", [])
+    applet_spec["runSpec"].setdefault("assetDepends", [])
+    if not dry_run:
+        region = dxpy.api.project_describe(dest_project, input_params={"fields": {"region": True}})["region"]
+
+        # if regionalOptions contain at least one region, they must include
+        # the region of the target project
+        if len(applet_spec.get('regionalOptions', {})) != 0 and region not in applet_spec.get('regionalOptions', {}):
+            err_mesg = "destination project is in region {} but \"regionalOptions\" do not contain this region. ".format(region)
+            err_mesg += "Please, update your \"regionalOptions\" specification"
+            raise AppBuilderException(err_mesg)
+
+        regional_options = applet_spec.get('regionalOptions', {}).get(region, {})
+
+        # We checked earlier that if region-specific values for the
+        # fields below are given, the same fields are not also specified
+        # in the top-level runSpec. So the operations below should not
+        # result in any user-supplied settings being clobbered.
+
+        if 'systemRequirements' in regional_options:
+            applet_spec["runSpec"]["systemRequirements"] = regional_options['systemRequirements']
+
+        if 'bundledDepends' in regional_options:
+            applet_spec["runSpec"]["bundledDepends"].extend(regional_options["bundledDepends"])
+        if 'assetDepends' in regional_options:
+            applet_spec["runSpec"]["assetDepends"].extend(regional_options["assetDepends"])
+
     # Inline Readme.md and Readme.developer.md
-    _inline_documentation_files(applet_spec, src_dir)
+    dxpy.executable_builder.inline_documentation_files(applet_spec, src_dir)
 
     # Inline the code of the program
-    if "runSpec" in applet_spec and "file" in applet_spec["runSpec"]:
-        # Avoid using runSpec.file for now, it's not fully implemented
-        #code_filename = os.path.join(src_dir, applet_spec["runSpec"]["file"])
-        #f = dxpy.upload_local_file(code_filename, wait_on_close=True)
-        #applet_spec["runSpec"]["file"] = f.get_id()
+    if "file" in applet_spec["runSpec"]:
         # Put it into runSpec.code instead
         with open(os.path.join(src_dir, applet_spec["runSpec"]["file"])) as code_fh:
             applet_spec["runSpec"]["code"] = code_fh.read()
             del applet_spec["runSpec"]["file"]
 
+    # If this is applet requires a cluster, inline any bootstrapScript code that may be provided.
+    # bootstrapScript is an *optional* clusterSpec parameter.
+    # NOTE: assumes bootstrapScript is always provided as a filename
+    if "systemRequirements" in applet_spec["runSpec"]:
+        sys_reqs = applet_spec["runSpec"]["systemRequirements"]
+        for entry_point in sys_reqs:
+            try:
+                bootstrap_script = os.path.join(src_dir, sys_reqs[entry_point]["clusterSpec"]["bootstrapScript"])
+                with open(bootstrap_script) as code_fh:
+                    sys_reqs[entry_point]["clusterSpec"]["bootstrapScript"] = code_fh.read()
+            except KeyError:
+                # either no "clusterSpec" or no "bootstrapScript" within "clusterSpec"
+                continue
+            except IOError:
+                raise AppBuilderException("The clusterSpec \"bootstrapScript\" could not be read.")
+
     # Attach bundled resources to the app
     if uploaded_resources is not None:
-        applet_spec["runSpec"].setdefault("bundledDepends", [])
         applet_spec["runSpec"]["bundledDepends"].extend(uploaded_resources)
 
+    # Validate and process assetDepends
+    asset_depends = applet_spec["runSpec"]["assetDepends"]
+    if type(asset_depends) is not list or any(type(dep) is not dict for dep in asset_depends):
+        raise AppBuilderException("Expected runSpec.assetDepends to be an array of objects")
+    for asset in asset_depends:
+        asset_project = asset.get("project", None)
+        asset_folder = asset.get("folder", '/')
+        if "id" in asset:
+            asset_record = dxpy.DXRecord(asset["id"]).describe(fields={'details'}, default_fields=True)
+        elif "name" in asset and asset_project is not None and "version" in asset:
+            try:
+                asset_record = dxpy.find_one_data_object(zero_ok=True, classname="record", typename="AssetBundle",
+                                                         name=asset["name"], properties=dict(version=asset["version"]),
+                                                         project=asset_project, folder=asset_folder, recurse=False,
+                                                         describe={"defaultFields": True, "fields": {"details": True}},
+                                                         state="closed", more_ok=False)
+            except dxpy.exceptions.DXSearchError:
+                msg = "Found more than one asset record that matches: name={0}, folder={1} in project={2}."
+                raise AppBuilderException(msg.format(asset["name"], asset_folder, asset_project))
+        else:
+            raise AppBuilderException("Each runSpec.assetDepends element must have either {'id'} or "
+                                      "{'name', 'project' and 'version'} field(s).")
+
+        if asset_record:
+            if "id" in asset:
+                asset_details = asset_record["details"]
+            else:
+                asset_details = asset_record["describe"]["details"]
+            if "archiveFileId" in asset_details:
+                archive_file_id = asset_details["archiveFileId"]
+            else:
+                raise AppBuilderException("The required field 'archiveFileId' was not found in "
+                                          "the details of the asset bundle %s " % asset_record["id"])
+            archive_file_name = dxpy.DXFile(archive_file_id).describe()["name"]
+            bundle_depends = {
+                "name": archive_file_name,
+                "id": archive_file_id
+            }
+            applet_spec["runSpec"]["bundledDepends"].append(bundle_depends)
+            # If the file is not found in the applet destination project, clone it from the asset project
+            if (not dry_run and
+                    dxpy.DXRecord(dxid=asset_record["id"], project=dest_project).describe()["project"] != dest_project):
+                dxpy.DXRecord(asset_record["id"], project=asset_record["project"]).clone(dest_project)
+        else:
+            raise AppBuilderException("No asset bundle was found that matched the specification %s"
+                                      % (json.dumps(asset)))
+
     # Include the DNAnexus client libraries as an execution dependency, if they are not already
     # there
     if dx_toolkit_autodep == "git":
@@ -392,8 +614,7 @@ def upload_applet(src_dir, uploaded_resources, check_name_collisions=True, overw
                           "url": "git://github.com/dnanexus/dx-toolkit.git",
                           "tag": "master",
                           "build_commands": "make install DESTDIR=/ PREFIX=/opt/dnanexus"}
-    # TODO: reject "beta" and "unstable" eventually
-    elif dx_toolkit_autodep in ("stable", "beta", "unstable"):
+    elif dx_toolkit_autodep == "stable":
         dx_toolkit_dep = {"name": "dx-toolkit", "package_manager": "apt"}
     elif dx_toolkit_autodep:
         raise AppBuilderException("dx_toolkit_autodep must be one of 'stable', 'git', or False; got %r instead" % (dx_toolkit_autodep,))
@@ -434,11 +655,10 @@ def upload_applet(src_dir, uploaded_resources, check_name_collisions=True, overw
     if archived_applet:
         archived_applet.set_properties({'replacedWith': applet_id})
 
-    # Now it is permissible to delete the old applet, if any
-    if applet_to_overwrite:
-        logger.info("Deleting applet %s" % (applet_to_overwrite,))
-        # TODO: test me
-        dxpy.DXProject(dest_project).remove_objects([applet_to_overwrite])
+    # Now it is permissible to delete the old applet(s), if any
+    if applets_to_overwrite:
+        logger.info("Deleting applet(s) %s" % (','.join(applets_to_overwrite)))
+        dxpy.DXProject(dest_project).remove_objects(applets_to_overwrite)
 
     return applet_id, applet_spec
 
@@ -482,18 +702,50 @@ def _update_version(app_name, version, app_spec, try_update=True):
             return None
         raise e
 
-def create_app(applet_id, applet_name, src_dir, publish=False, set_default=False, billTo=None, try_versions=None, try_update=True, confirm=True):
+
+def create_app_multi_region(regional_options, app_name, src_dir, publish=False, set_default=False, billTo=None,
+                            try_versions=None, try_update=True, confirm=True):
+    """
+    Creates a new app object from the specified applet(s).
+
+    :param regional_options: Region-specific options for the app. See
+        https://wiki.dnanexus.com/API-Specification-v1.0.0/Apps#API-method:-/app/new
+        for details; this should contain keys for each region the app is
+        to be enabled in, and for the values, a dict containing (at
+        minimum) a key "applet" whose value is an applet ID for that
+        region.
+    :type regional_options: dict
+    """
+    return _create_app(dict(regionalOptions=regional_options), app_name, src_dir, publish=publish,
+                       set_default=set_default, billTo=billTo, try_versions=try_versions, try_update=try_update,
+                       confirm=confirm)
+
+
+def create_app(applet_id, applet_name, src_dir, publish=False, set_default=False, billTo=None, try_versions=None,
+               try_update=True, confirm=True, regional_options=None):
     """
     Creates a new app object from the specified applet.
-    """
-    app_spec = _get_app_spec(src_dir)
-    logger.info("Will create app with spec: %s" % (app_spec,))
 
-    app_spec["applet"] = applet_id
-    app_spec["name"] = applet_name
+    .. deprecated:: 0.204.0
+       Use :func:`create_app_multi_region()` instead.
+
+    """
+    # In this case we don't know the region of the applet, so we use the
+    # legacy API {"applet": applet_id} without specifying a region
+    # specifically.
+    return _create_app(dict(applet=applet_id), applet_name, src_dir, publish=publish, set_default=set_default,
+                       billTo=billTo, try_versions=try_versions, try_update=try_update, confirm=confirm)
+
+
+def _create_app(applet_or_regional_options, app_name, src_dir, publish=False, set_default=False, billTo=None,
+                try_versions=None, try_update=True, confirm=True):
+    app_spec = _get_app_spec(src_dir)
+    logger.info("Will create app with spec: %s" % (json.dumps(app_spec),))
+
+    app_spec.update(applet_or_regional_options, name=app_name)
 
     # Inline Readme.md and Readme.developer.md
-    _inline_documentation_files(app_spec, src_dir)
+    dxpy.executable_builder.inline_documentation_files(app_spec, src_dir)
 
     if billTo:
         app_spec["billTo"] = billTo
@@ -653,8 +905,77 @@ def create_app(applet_id, applet_name, src_dir, publish=False, set_default=False
         # If no versions of this app have ever been published, then
         # we'll set the "default" tag to point to the latest
         # (unpublished) version.
-        no_published_versions = len(list(dxpy.find_apps(name=applet_name, published=True, limit=1))) == 0
+        no_published_versions = len(list(dxpy.find_apps(name=app_name, published=True, limit=1))) == 0
         if no_published_versions:
             dxpy.api.app_add_tags(app_id, input_params={'tags': ['default']})
 
     return app_id
+
+
+def assert_consistent_regions(from_app_spec, from_command_line):
+    """
+    :param from_app_spec: The regional options specified in dxapp.json.
+    :type from_app_spec: dict or None.
+    :param from_command_line: The regional options specified on the
+    command-line via --region.
+    :type from_command_line: list or None.
+    """
+    if from_app_spec is None or from_command_line is None:
+        return
+    if set(from_app_spec) != set(from_command_line):
+        raise dxpy.app_builder.AppBuilderException("--region and the 'regionalOptions' key in dxapp.json do not agree")
+
+
+def get_enabled_regions(app_spec, from_command_line):
+    """Returns a list of the regions in which the app should be enabled.
+
+    Also validates that app_spec['regionalOptions'], if supplied, is
+    well-formed.
+
+    :param app_spec: app specification
+    :type app_spec: dict
+    :param from_command_line: The regions specified on the command-line
+      via --region
+    :type from_command_line: list or None
+
+    """
+    from_app_spec = app_spec.get('regionalOptions')
+
+    if from_app_spec is not None:
+        if not isinstance(from_app_spec, dict):
+            raise dxpy.app_builder.AppBuilderException("The field 'regionalOptions' in dxapp.json must be a mapping")
+        if not from_app_spec:
+            raise dxpy.app_builder.AppBuilderException(
+                "The field 'regionalOptions' in dxapp.json must be a non-empty mapping")
+        regional_options_list = list(from_app_spec.items())
+        for region, opts_for_region in regional_options_list:
+            if not isinstance(opts_for_region, dict):
+                raise dxpy.app_builder.AppBuilderException("The field 'regionalOptions['" + region +
+                                                           "']' in dxapp.json must be a mapping")
+            if set(opts_for_region.keys()) != set(regional_options_list[0][1].keys()):
+                if set(opts_for_region.keys()) - set(regional_options_list[0][1].keys()):
+                    with_key, without_key = region, regional_options_list[0][0]
+                    key_name = next(iter(set(opts_for_region.keys()) - set(regional_options_list[0][1].keys())))
+                else:
+                    with_key, without_key = regional_options_list[0][0], region
+                    key_name = next(iter(set(regional_options_list[0][1].keys()) - set(opts_for_region.keys())))
+                raise dxpy.app_builder.AppBuilderException(
+                    "All regions in regionalOptions must specify the same options; " +
+                    "%s was given for %s but not for %s" % (key_name, with_key, without_key)
+                )
+            for key in opts_for_region:
+                if key in app_spec.get('runSpec', {}):
+                    raise dxpy.app_builder.AppBuilderException(
+                        key + " cannot be given in both runSpec and in regional options for " + region)
+
+    assert_consistent_regions(from_app_spec, from_command_line)
+
+    enabled_regions = None
+    if from_app_spec is not None:
+        enabled_regions = from_app_spec.keys()
+    elif from_command_line is not None:
+        enabled_regions = from_command_line
+
+    if enabled_regions is not None and len(enabled_regions) == 0:
+        raise AssertionError("This app should be enabled in at least one region")
+    return enabled_regions
