@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2015 DNAnexus, Inc.
+# Copyright (C) 2013-2016 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -23,16 +23,17 @@ from __future__ import print_function, unicode_literals, division, absolute_impo
 # TODO: refactor all dx run helper functions here
 
 import os, sys, json, collections, pipes
+from ..bindings.dxworkflow import DXWorkflow
 
 import dxpy
 from . import INTERACTIVE_CLI
 from ..exceptions import DXCLIError, DXError
 from ..utils.printing import (RED, GREEN, WHITE, BOLD, ENDC, UNDERLINE, fill)
-from ..utils.describe import (get_find_executions_string, get_ls_l_desc, parse_typespec)
-from ..utils.resolver import (get_first_pos_of_char, is_hashid, is_job_id, is_localjob_id, paginate_and_pick, pick,
+from ..utils.describe import (get_find_executions_string, get_ls_l_desc, get_ls_l_desc_fields, parse_typespec)
+from ..utils.resolver import (parse_input_keyval, is_hashid, is_job_id, is_localjob_id, paginate_and_pick, pick,
                               resolve_existing_path, resolve_multiple_existing_paths, split_unescaped, is_analysis_id)
 from ..utils import OrderedDefaultdict
-from ..compat import input, str, shlex, basestring
+from ..compat import input, str, shlex, basestring, USING_PYTHON2
 
 ####################
 # -i Input Parsing #
@@ -144,7 +145,7 @@ def interactive_help(in_class, param_desc, prompt):
             if opt_num in range(3):
                 result_generator = dxpy.find_data_objects(classname=in_class,
                                                           typename=param_desc.get('type'),
-                                                          describe=True,
+                                                          describe=dict(fields=get_ls_l_desc_fields()),
                                                           project=query_project)
                 print('\nAvailable data:')
                 result_choice = paginate_and_pick(result_generator,
@@ -397,8 +398,8 @@ def get_input_single(param_desc):
                 print(fill('Error occurred when parsing for class ' + in_class + ': ' + str(details)))
                 continue
             if 'choices' in param_desc and value not in param_desc['choices']:
-                print(fill(RED() + BOLD() + 'Warning:' + ENDC() + ' value "' + str(value) + '" for input ' + WHITE()
-                           + BOLD() + param_desc['name'] + ENDC() + ' is not in the list of choices for that input'))
+                print(fill(RED() + BOLD() + 'Warning:' + ENDC() + ' value "' + str(value) + '" for input ' + WHITE() +
+                           BOLD() + param_desc['name'] + ENDC() + ' is not in the list of choices for that input'))
             return value
     except EOFError:
         raise DXCLIError('Unexpected end of input')
@@ -407,12 +408,28 @@ def get_optional_input_str(param_desc):
     return param_desc.get('label', param_desc['name']) + ' (' + param_desc['name'] + ')'
 
 class ExecutableInputs(object):
-    def __init__(self, executable=None, input_name_prefix=None, input_spec=None):
+    def __init__(self, executable=None, input_name_prefix=None, input_spec=None, active_region=None):
+        """
+        :param executable: Executable object handler
+        :type executable: :class:`~dxpy.bindings.dxapplet.DXApplet`,
+                          :class:`~dxpy.bindings.dxapp.DXApp`,
+                          :class:`~dxpy.bindings.dxworkflow.DXWorkflow`,
+                          :class:`~dxpy.bindings.dxglobalworkflow.DXGlobalWorkflow`
+        :param input_name_prefix: A prefix set on an input field name
+        :type input_name_prefix: string
+        :param input_spec: Input specification
+        :type input_spec: dict
+        :param active_region: The region in which the executable is run, determined by the destination project context.
+        :type active_region: string
+        """
         self.executable = executable
-        self._desc = {} if self.executable is None else executable.describe()
-        self.input_spec = collections.OrderedDict() if 'inputSpec' in self._desc or input_spec else None
+        self.region = active_region
+
+        self._desc = self.get_executable_description()
+
         self.required_inputs, self.optional_inputs, self.array_inputs = [], [], set()
         self.input_name_prefix = input_name_prefix
+        self.inputs = OrderedDefaultdict(list)
 
         # List of tuples (input name, input value, input class, index), where input name and input value are
         # propagated from command-line (input class is propagated from the input spec, and may be None if no input
@@ -422,27 +439,55 @@ class ExecutableInputs(object):
         # single input value instead of a list of input values, then index is -1.
         self.requires_resolution = []
 
+        # update input_spec passed to the constructor and initialize
+        # self.input_spec, self.optional_inputs, self.required_inputs, self.array_inputs
+        self.input_spec = collections.OrderedDict() if 'inputSpec' in self._desc or input_spec else None
+
         if input_spec is None:
             input_spec = self._desc.get('inputSpec', [])
 
-        if input_spec is None and self._desc['class'] == 'workflow':
+        if input_spec is None and self._desc['class'] in ('workflow', 'globalworkflow'):
             # this is only the case if it's a workflow with an
             # inaccessible stage
             inaccessible_stages = [stage['id'] for stage in self._desc['stages'] if stage['accessible'] is False]
             raise DXCLIError('The workflow ' + self._desc['id'] + ' has the following inaccessible stage(s): ' + ', '.join(inaccessible_stages))
 
+        # Workflow-level inputs (defined in "inputs")
+        #  i. If the workflow has no "inputs"
+        #   * The inputs can be passed to stages directly
+        # ii. If the workflow has "inputs" (in a closed or open state)
+        #   * Only inputs defined in inputs can be passed to the workflow,
+        #     using workflow-level input names
+        if self._accept_only_workflow_level_inputs():
+            input_spec = self._desc.get('inputs', [])
+
         for spec_atom in input_spec:
+            input_name = spec_atom['name']
             if spec_atom['class'].startswith('array:'):
-                self.array_inputs.add(spec_atom['name'])
-            self.input_spec[spec_atom['name']] = spec_atom
-            if "default" in spec_atom or spec_atom.get("optional") == True:
-                self.optional_inputs.append(spec_atom['name'])
+                self.array_inputs.add(input_name)
+            self.input_spec[input_name] = spec_atom
+            if self._is_input_optional(spec_atom):
+                self.optional_inputs.append(input_name)
             else:
-                self.required_inputs.append(spec_atom['name'])
+                self.required_inputs.append(input_name)
+
+    def _accept_only_workflow_level_inputs(self):
+        return self._desc.get('inputs') is not None
+
+    def get_executable_description(self):
+        if self.executable is None:
+            return {}
+        elif isinstance(self.executable, dxpy.DXGlobalWorkflow):
+            global_workflow_desc = self.executable.describe()
+            return self.executable.append_underlying_workflow_desc(global_workflow_desc, self.region)
+        else:
+            return self.executable.describe()
 
-        self.inputs = OrderedDefaultdict(list)
 
     def update(self, new_inputs, strip_prefix=True):
+        """
+        Updates the inputs dictionary with the key/value pairs from new_inputs, overwriting existing keys.
+        """
         if strip_prefix and self.input_name_prefix is not None:
             for i in new_inputs:
                 if i.startswith(self.input_name_prefix):
@@ -451,6 +496,10 @@ class ExecutableInputs(object):
             self.inputs.update(new_inputs)
 
     def _update_requires_resolution_inputs(self):
+        """
+        Updates self.inputs with resolved input values (the input values that were provided
+        as paths to items that require resolutions, eg. folder or job/analyses ids)
+        """
         input_paths = [quad[1] for quad in self.requires_resolution]
         results = resolve_multiple_existing_paths(input_paths)
         for input_name, input_value, input_class, input_index in self.requires_resolution:
@@ -549,7 +598,10 @@ class ExecutableInputs(object):
             else:
                 try:
                     parsed_input_value = json.loads(input_value, object_pairs_hook=collections.OrderedDict)
-                    if type(parsed_input_value) not in (collections.OrderedDict, list, int, long, float):
+                    immediate_types = {collections.OrderedDict, list, int, float}
+                    if USING_PYTHON2:
+                        immediate_types.add(long) # noqa
+                    if type(parsed_input_value) not in immediate_types:
                         raise Exception()
                 except:
                     # Not recognized JSON (list or dict), so resolve it as a name
@@ -625,7 +677,7 @@ class ExecutableInputs(object):
 
             readline.set_completer_delims("")
 
-            readline.write_history_file(os.path.expanduser('~/.dnanexus_config/.dx_history'))
+            readline.write_history_file(os.path.join(dxpy.config.get_user_conf_dir(), '.dx_history'))
             readline.clear_history()
             readline.set_completer()
         except:
@@ -633,6 +685,7 @@ class ExecutableInputs(object):
 
     def uninit_completer(self):
         try:
+            import readline
             readline.set_completer()
             readline.clear_history()
         except:
@@ -718,16 +771,11 @@ class ExecutableInputs(object):
 
         if args.input is not None:
             for keyeqval in args.input:
-                try:
-                    first_eq_pos = get_first_pos_of_char('=', keyeqval)
-                    if first_eq_pos == -1:
-                        raise
-                    name = split_unescaped('=', keyeqval)[0]
-                    value = keyeqval[first_eq_pos + 1:]
-                except:
-                    raise DXCLIError('An input was found that did not conform to the syntax: -i<input name>=<input value>')
-                self.add(self.executable._get_input_name(name) if \
-                         self._desc.get('class') == 'workflow' else name, value)
+                name, value = parse_input_keyval(keyeqval)
+                if '.' in name and self._accept_only_workflow_level_inputs():
+                    raise DXCLIError('The input with a key '+ name + ' was passed to a stage but the workflow accepts inputs only on the workflow level')
+                self.add(self.executable._get_input_name(name, region=args.region, describe_output=self._desc) if \
+                         self._desc.get('class') in ('workflow', 'globalworkflow') else name, value)
             self._update_requires_resolution_inputs()
 
         if self.input_spec is None:
@@ -745,12 +793,5 @@ class ExecutableInputs(object):
                 if missing_required_inputs:
                     raise DXCLIError('Some inputs (%s) are missing, and interactive mode is not available' % (', '.join(missing_required_inputs)))
 
-        # if self.required_input_specs is not None and (len(self.required_input_specs) > 0 or len(self.optional_input_specs) > 0):
-        #     if sys.stdout.isatty():
-        #         self.prompt_for_missing()
-        # elif self.required_input_specs is not None:
-        #     if not args.brief:
-        #         print fill('No input given, and applet/app takes in no inputs.  Skipping interactive mode for input selection.')
-        # else:
-        #     if not args.brief:
-        #         print fill('No input given, and applet has no input specification.  Skipping interactive mode for input selection (no input parameters will be set).  To provide input parameters anyway, please specify them explicitly using one of the input flags.')
+    def _is_input_optional(self, spec_atom):
+        return spec_atom.get("optional") == True or 'default' in spec_atom
