@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2015 DNAnexus, Inc.
+# Copyright (C) 2013-2016 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -260,8 +260,30 @@ instance_type_arg.add_argument('--instance-type-help',
                                help=fill('Print help for specifying instance types'),
                                action=PrintInstanceTypeHelp)
 
+property_args = argparse.ArgumentParser(add_help=False)
+property_args.add_argument('--property', dest='properties', metavar='KEY=VALUE',
+                           help=(fill('Key-value pair to add as a property; repeat as necessary,',
+                                      width_adjustment=-24) + '\n' +
+                                 fill('e.g. "--property key1=val1 --property key2=val2"',
+                                      width_adjustment=-24, initial_indent=' ', subsequent_indent=' ',
+                                      break_on_hyphens=False)),
+                           action='append')
+
+tag_args = argparse.ArgumentParser(add_help=False)
+tag_args.add_argument('--tag', metavar='TAG', dest='tags',
+                      help=fill('Tag for the resulting execution; repeat as necessary,', width_adjustment=-24) + '\n' +
+                      fill('e.g. "--tag tag1 --tag tag2"', width_adjustment=-24,
+                           break_on_hyphens=False, initial_indent=' ', subsequent_indent=' '),
+                      action='append')
+
+contains_phi = argparse.ArgumentParser(add_help=False)
+contains_phi.add_argument('--phi', dest='containsPHI', choices=["true", "false"],
+                          help='If set to true, only projects that contain PHI data will be retrieved. ' +
+                          'If set to false, only projects that do not contain PHI data will be retrieved.')
+
 def _parse_inst_type(thing):
     if thing.strip().startswith('{'):
+        # expects a map of entry point to instance type
         try:
             return json.loads(thing)
         except ValueError:
@@ -269,6 +291,7 @@ def _parse_inst_type(thing):
     else:
         return thing
 
+
 def process_instance_type_arg(args, for_workflow=False):
     if args.instance_type:
         if for_workflow:
@@ -288,3 +311,32 @@ def process_instance_type_arg(args, for_workflow=False):
         else:
             # is a string
             args.instance_type = _parse_inst_type(args.instance_type)
+
+
+def get_update_project_args(args):
+    input_params = {}
+    if args.name is not None:
+        input_params["name"] = args.name
+    if args.summary is not None:
+        input_params["summary"] = args.summary
+    if args.description is not None:
+        input_params["description"] = args.description
+    if args.protected is not None:
+        input_params["protected"] = True if args.protected == 'true' else False
+    if args.restricted is not None:
+        input_params["restricted"] = True if args.restricted == 'true' else False
+    if args.download_restricted is not None:
+        input_params["downloadRestricted"] = True if args.download_restricted == 'true' else False
+    if args.containsPHI is not None:
+        input_params["containsPHI"] = True if args.containsPHI == 'true' else False
+    if args.bill_to is not None:
+        input_params["billTo"] = args.bill_to
+    return input_params
+
+
+def process_phi_param(args):
+    if args.containsPHI is not None:
+        if args.containsPHI == "true":
+            args.containsPHI = True
+        elif args.containsPHI == "false":
+            args.containsPHI = False
