@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2015 DNAnexus, Inc.
+# Copyright (C) 2013-2016 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -20,6 +20,7 @@ This submodule gives basic utilities for printing to the terminal.
 
 import textwrap, subprocess, os, sys
 import json
+import platform
 from ..compat import USING_PYTHON2, sys_encoding
 from ..exceptions import DXCLIError
 
@@ -44,6 +45,12 @@ def CYAN(message=None):
     else:
         return CYAN() + message + ENDC()
 
+def LIGHTBLUE(message=None):
+    if message is None:
+        return '\033[1;34m' if color_state else ''
+    else:
+        return LIGHTBLUE() + message + ENDC()
+
 def BLUE(message=None):
     if message is None:
         return '\033[34m' if color_state else ''
@@ -131,8 +138,16 @@ def pager(content, pager=None, file=None):
         if tty_rows > content_rows and tty_cols > content_cols:
             raise DXCLIError() # Just print the content, don't use a pager
 
-        pager_process = subprocess.Popen(pager or os.environ.get('PAGER', 'less -RS'),
-                                         shell=True, stdin=subprocess.PIPE, stdout=file)
+        if pager is None:
+            pager = os.environ.get('PAGER', 'less -RS')
+        if platform.system() == 'Windows':
+            # Verify if the pager is available on Windows
+            try:
+                subprocess.call(pager)
+            except:
+                raise DXCLIError()  # Just print the content, don't use a pager
+
+        pager_process = subprocess.Popen(pager, shell=True, stdin=subprocess.PIPE, stdout=file)
         pager_process.stdin.write(content.encode(sys_encoding))
         pager_process.stdin.close()
         pager_process.wait()
@@ -164,6 +179,18 @@ def _format_find_projects_results(results):
               DELIMITER(' (') + result["level"] + DELIMITER(')'))
 
 
+def _format_find_apps_results(results, verbose=False):
+    def maybe_x(result):
+        return DNANEXUS_X() if result['describe']['billTo'] == 'org-dnanexus' else ' '
+
+    if not verbose:
+        for result in results:
+            print(maybe_x(result) + DELIMITER(" ") + result['describe'].get('title', result['describe']['name']) + DELIMITER(' (') + result["describe"]["name"] + DELIMITER("), v") + result["describe"]["version"])
+    else:
+        for result in results:
+            print(maybe_x(result) + DELIMITER(" ") + result["id"] + DELIMITER(" ") + result['describe'].get('title', result['describe']['name']) + DELIMITER(' (') + result["describe"]["name"] + DELIMITER('), v') + result['describe']['version'] + DELIMITER(" (") + ("published" if result["describe"].get("published", 0) > 0 else "unpublished") + DELIMITER(")"))
+
+
 def _format_find_org_members_results(results):
     for result in results:
         print(result["id"] + DELIMITER(" : ") + result['describe']['first'] + DELIMITER(' ') +
@@ -175,7 +202,8 @@ def format_find_results(args, results):
     """
     Formats the output of ``dx find ...`` commands for `--json` and `--brief` arguments; also formats if no formatting
     arguments are given.
-    Currently used for ``dx find projects``, ``dx find org_projects``, and ``dx find org_members``
+    Currently used for ``dx find projects``, ``dx find org_projects``, ``dx find org_apps``,
+    and ``dx find org_members``
     """
     if args.json:
         print(json.dumps(list(results), indent=4))
@@ -185,5 +213,7 @@ def format_find_results(args, results):
     else:
         if args.func.__name__ in ("find_projects", "org_find_projects"):
             _format_find_projects_results(results)
-        if args.func.__name__ in ("org_find_members"):
+        elif args.func.__name__ in ("org_find_members"):
             _format_find_org_members_results(results)
+        elif args.func.__name__ in ("org_find_apps"):  # should have "find_apps" here one day
+            _format_find_apps_results(results, verbose=args.verbose)
