@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-# Copyright (C) 2013-2015 DNAnexus, Inc.
+# Copyright (C) 2013-2016 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -21,6 +21,7 @@ from tempfile import NamedTemporaryFile, mkdtemp
 
 import dxpy
 import dxpy_testutil as testutil
+from dxpy.exceptions import DXError
 
 # TODO: unit tests for dxpy.utils.completer
 
@@ -98,8 +99,13 @@ class TestDXTabCompletion(unittest.TestCase):
         dxpy.set_workspace_id(self.project_id)
 
     def tearDown(self):
-        dxpy.api.project_remove_folder(self.project_id,
-                                       {"folder": "/", "recurse": True})
+        completed = False
+        while not completed:
+            resp = dxpy.api.project_remove_folder(self.project_id,
+                                                  {"folder": "/", "recurse": True, "partial": True})
+            if 'completed' not in resp:
+                raise DXError('Error removing folder')
+            completed = resp['completed']
         for var in 'IFS', '_ARGCOMPLETE', '_DX_ARC_DEBUG', 'COMP_WORDBREAKS':
             if var in os.environ:
                 del os.environ[var]
@@ -155,7 +161,9 @@ class TestDXTabCompletion(unittest.TestCase):
 
     def test_applet_completion(self):
         dxapplet = dxpy.DXApplet()
-        dxapplet.new(runSpec={"code": "placeholder", "interpreter": "bash"},
+        run_spec = {"code": "placeholder", "interpreter": "bash",
+                    "distribution": "Ubuntu", "release": "14.04"}
+        dxapplet.new(runSpec=run_spec,
                      dxapi="1.0.0",
                      name="my applet")
 
@@ -165,7 +173,7 @@ class TestDXTabCompletion(unittest.TestCase):
         self.assert_completion("dx ls ", "my applet ")
 
         # not available to run when hidden
-        dxapplet.new(runSpec={"code": "placeholder", "interpreter": "bash"},
+        dxapplet.new(runSpec=run_spec,
                      dxapi="1.0.0",
                      name="hidden",
                      hidden=True)
